@@ -48,6 +48,14 @@ setLocalTranslate('cl_lib3', sys.modules[__name__])
 reVerSplit = re.compile(r"^(?:.*/var/db/pkg/)?(?:(\w+-\w+)/)?(.*?)-(([^-]+?)"
                         "(?:-(r\d+))?)(?:.(tbz2))?$", re.S)
 
+class RepositoryPath(object):
+    Layman = '/var/lib/layman'
+    Calculate = path.join(Layman, "calculate")
+    Distros = path.join(Layman, "distros")
+    Gentoo = '/usr/portage'
+    CalculateProfiles = path.join(Calculate, "profiles")
+    CalculateTemplates = path.join(CalculateProfiles, "templates")
+
 def reVerSplitToPV(x):
     """Convert match from reVerSplit to PV hash"""
     if type(x) in (str, unicode):
@@ -109,6 +117,7 @@ def isPkgInstalled(pkg, prefix='/', sortByVersion=False):
                 if pkg_info['PN'] == pkgname:
                     pkg_info.update({'CATEGORY': category})
                     yield pkg_info
+
         res = list(package_generator(path.join(pkgDir, category)))
         if len(res) > 1 and sortByVersion:
             return sorted(res, key=lambda x: getTupleVersion(x['PVR']))
@@ -138,6 +147,25 @@ def getPkgActiveUses(fullpkg):
     return list(set(res[0]) & set(res[1]))
 
 
+def getInstalledAtom(atom, prefix='/'):
+    atom = EmergePackage(atom)
+    db_path = path.join(prefix, 'var/db/pkg')
+    glob_fn = path.join(db_path, "%s-*" % atom["CATEGORY/PN"])
+    for pkg_dn in glob.glob(glob_fn):
+        slot = readFile(path.join(pkg_dn, "SLOT"))
+        # удаляем subslot
+        slot = slot.partition("/")[0]
+        if not slot.strip():
+            continue
+        find_atom = EmergePackage("%s:%s" % (pkg_dn[len(db_path) + 1:], slot))
+        if find_atom["CATEGORY/PN"] == atom["CATEGORY/PN"]:
+            if atom["SLOT!"] and atom["SLOT"] != find_atom["SLOTONLY"]:
+                continue
+            if atom["PN"] != atom["PF"] and find_atom["PF"] != atom["PF"]:
+                continue
+            yield find_atom
+
+
 def getSquashList():
     """Get supprted squashfs compressions method"""
     wantMethod = {"lzo", "lzma", "xz", "gzip"}
@@ -216,7 +244,7 @@ def searchProfile(dirpath, configname, repository_sub=None):
     return list(search(dirpath))
 
 
-class Layman:
+class Layman(object):
     """
     Объект для управления репозиториями Layman
 
@@ -434,7 +462,7 @@ class EmergePackage(Mapping):
             slot=slot, repo=repo))
 
     attrs = ('CATEGORY', 'PN', 'PF', 'SLOT', 'REPO', 'P', 'PV', 'PR', 'PVR',
-             'CATEGORY/PN')
+             'CATEGORY/PN', 'SLOT!', 'SLOTONLY')
 
     def _parsePackageString(self, s):
         """
@@ -450,6 +478,8 @@ class EmergePackage(Mapping):
                  'PF': x[PF],
                  'P': x[P],
                  'SLOT': x[SLOT] or '0',
+                 'SLOTONLY': (x[SLOT] or '0').partition('/')[0],
+                 'SLOT!': x[SLOT] or '',
                  'REPO': x[REPO] or self.default_repo,
                  'CATEGORY/PN': "%s/%s" % (x[CATEGORY], x[PN]),
                  'PR': x[PR] or 'r0'}
@@ -1052,9 +1082,10 @@ class SimpleRepositoryMapper(Mapping):
     """
     Определение пути до репозитория
     """
-    map_repository = {'gentoo': '/usr/portage',
-                      'calculate': '/var/lib/layman/calculate'}
-    layman_path = '/var/lib/layman'
+    map_repository = {'gentoo': RepositoryPath.Gentoo,
+                      'calculate': RepositoryPath.Calculate,
+                      'distros': RepositoryPath.Distros}
+    layman_path = RepositoryPath.Layman
 
     def __init__(self, prefix='/'):
         self.prefix = prefix
@@ -1163,3 +1194,15 @@ class InstalledPackageInfo(object):
 
     def __repr__(self):
         return "InstalledPackageInfo(%s)" % self.atom
+
+
+def makeCfgName(origfilename):
+    """
+    Сгенерировать имя cfg0000 для файла
+    """
+    directory, filename = path.split(origfilename)
+    for i in range(0, 9999):
+        fn = path.join(directory, "._cfg%04d_%s" % (i, filename))
+        if not path.exists(fn):
+            return fn
+    return origfilename
